@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005 Marcin R. Wrochniak <wrochniak@gmail.com>
+  Copyright (C) 2005-2006 Marcin R. Wrochniak <wrochniak@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2 as
@@ -47,20 +47,48 @@
 
 #include "column.h"
 
-char storing;
+/* Command that's actually being executed. */
+char * actual_command;
 
-char * aptcmd;
-
-char use_realcmd;
+bool use_realcmd;
 
+/* Input for commands executed from realizecmd(). */
 char * yes;
 
-// Number of steps in commitlog
-int commit_count;
-char ** commitz;
+/* Command queue mode - just saving commands to command queue. */
+bool command_queue_mode;
+/* Size of command queue. */
+int command_queue_count;
+/* Items in command queue. */
+char ** command_queue_items;
 
-extern void i_setsig();
 
+/* We should ignore SIGPIPE.
+ * If we don't, aptsh is going to terminate when user creates a bad pipe. 
+ * Example:
+ * rls | less
+ * and user terminates 'less' before it reaches end of output.
+ */
+void i_setsig()
+{
+	static char set = 0;
+	struct sigaction act;
+	if (! set) {
+		act.sa_handler = SIG_IGN;
+		act.sa_flags = 0;
+		sigaction(SIGPIPE, &act, NULL);
+		set = 1;
+	}
+}
+
+/* Prefix of executables in libexec directory (aptsh_{ls,rls,printer}). */
+#define LIBEXEC_PREFIX "aptsh_"
+
+
+/* Little helper for commands that execute external command.
+ * If 'use_realcmd' is true, intput of executed command will be feeded by 
+ * content of 'yes'. Otherwise, command will be just executed from system().
+ */
 void realizecmd(char * sth) {
 	if (use_realcmd) {
 		i_setsig();
@@ -78,9 +106,23 @@ void realizecmd(char * sth) {
 	}
 }
 
+
+/* This function executes actual_command command (ie. install aptsh),
+ * preceding it with shell command - 'exec' (ie. apt-get).
+ */
+void newcmd(char *exec)
+{
+	char * tmp;
+	tmp = (char*)malloc(strlen(actual_command)+strlen(exec)+2);
+	sprintf(tmp, "%s %s", exec, actual_command);
+	realizecmd(tmp);
+	free(tmp);
+}
+
 long tmpdate;
 
-// Checks whether cache file has been changed since last reading (stored in tmpdate)
+/* Checks whether cache file has been changed since last reading (stored in tmpdate). */
+/* TODO: It's possible that it's not needed any more, needs checking. */
 void check_a()
 {
 	if (!CFG_REFRESH_INDEXES && !CFG_REFRESH_INDEXES_ALL)
@@ -147,10 +189,11 @@ struct command cmds[] = {
 	{ "quit", NULL, NONE, false, NULL, false, false },
 };
 
-// Check whether package exists
-// ex decides which packages ought to be checked:
-//   AVAILABLE - all packages (default)
-//   INSTALLED  - only installed packages
+/* Check whether package exists.
+ * 'ex' decides which packages ought to be checked:
+ *   AVAILABLE - all packages (default)
+ *   INSTALLED - only installed packages.
+ */
 bool package_exists(char * name, enum completion ex = AVAILABLE)
 {
 	static pkgCache * Cache;
@@ -161,7 +204,7 @@ bool package_exists(char * name, enum completion ex = AVAILABLE)
 	for (e = Cache->PkgBegin(); e.end() == false; e++) {
 		if (ex == INSTALLED) {
 			pkgCache::Package * ppk = (pkgCache::Package *)e;
-			// 6 means installed
+			/* 6 means installed. */
 			if (ppk->CurrentState == 6) {
 				if (! strcmp(e.Name(), name)) {
 					return true;
@@ -178,16 +221,19 @@ bool package_exists(char * name, enum completion ex = AVAILABLE)
 	return false;
 }
 
-// Validates aptsh commadn
-// You can't give a command with ';' at the begining, if so it will throw up a warning
-// Also, command must be already trimmed
+/* Validates Aptsh command. */
+/* Command must be already trimmed. */
 int validate(char * cmd)
 {
-	//cmd = trimleft(cmd); <- it should be trimmed in execute()
+#if 0
+	/* It should be already trimmed in execute(). */
+	//cmd = trimleft(cmd);
+#endif
 
-	// Well, we don't try to validate shell commands
+	/* Well, we don't try to validate shell commands. */
 	if (cmd[0] == '.')
 		return 0;
+
 	char * tmp = first_word(cmd);
 	char ok = 0;
 	enum completion sort = NONE;
@@ -218,7 +264,7 @@ int validate(char * cmd)
 			if (! strcmp(tmp, ""))
 				break;
 
-			// it may be a parameter for apt, not a pkg's name
+			/* It may be a parameter for apt, not a pkg's name. */
 			if (tmp[0] == '-') {
 				cmd2 = cmd2+strlen(tmp);
 				continue;
@@ -238,7 +284,7 @@ int validate(char * cmd)
 			if (! strcmp(tmp, ""))
 				break;
 			
-			// it may be a parameter for apt, not a pkg's name
+			/* It may be a parameter for apt, not a pkg's name. */
 			if (tmp[0] == '-') {
 				cmd2 = cmd2+strlen(tmp);
 				continue;
@@ -255,20 +301,14 @@ int validate(char * cmd)
 	return 0;
 }
 
-// This macro executes aptcmd command (ie. install aptsh), preceding it with shell command - WW (ie. apt-get)
-char * tmp;
-#define newcmd(WW) \
-tmp = (char*)malloc(strlen(aptcmd)+strlen(WW)+2);\
-sprintf(tmp, "%s %s", WW, aptcmd);\
-realizecmd(tmp);
-
-// Returns >0 when user wants to exit
+/* Returns >0 when user wants to exit. */
 int execute(char * line, char addhistory)
 {
 	if (CFG_USE_HISTORY && addhistory)
 		if (line && strcmp("", trimleft(line)) &&
-		/* below it check whether history is empty, if so it allows to add new entry, else it checks
-		   whether this line was added recently - if not, it allows to add n.e. */
+		/* Below it checks whether history is empty, if so it allows to add new entry,
+		 * else it checks whether this line was added recently - if not, it allows to add n.e.
+		 */
 		( history_list() == NULL ? 1 : strcmp(history_list()[history_length-1]->line, line) )){
 			add_history(line);
 			if (CFG_HISTORY_COUNT)
@@ -279,11 +319,11 @@ int execute(char * line, char addhistory)
 		}
 	
 	if (trimleft(line)[0] == '`') {
-		storing = !storing;
+		command_queue_mode = !command_queue_mode;
 		return 0;
 	}
 
-	if (storing) {
+	if (command_queue_mode) {
 		char * line_t = trimleft(trimleft(line));
 		
 		if (CFG_QUEUE_SIMULATE) {
@@ -292,7 +332,7 @@ int execute(char * line, char addhistory)
 			for (int i = 0; i < CMD_NUM; i++) {
 				if (!strcmp(fword, cmds[i].name) && cmds[i].apt_get) {
 					const char * simulator = "apt-get --simulate";
-					tmp = (char*)malloc(strlen(line)+strlen(simulator)+2);
+					char *tmp = (char*)malloc(strlen(line)+strlen(simulator)+2);
 					sprintf(tmp, "%s %s", simulator, line);
 					system(tmp);
 					free(tmp);
@@ -302,12 +342,12 @@ int execute(char * line, char addhistory)
 			free(fword);
 		}
 		
-		commit_count++;
-		commitz = (char**)realloc(commitz, commit_count*sizeof(char*));
+		command_queue_count++;
+		command_queue_items = (char**)realloc(command_queue_items, command_queue_count*sizeof(char*));
 
 		validate(line_t);
 		
-		commitz[commit_count-1] = strdup(line_t);
+		command_queue_items[command_queue_count-1] = strdup(line_t);
 		return 0;
 	}
 
@@ -323,7 +363,7 @@ int execute(char * line, char addhistory)
 		return 1;
 		
 	
-	aptcmd = line;
+	actual_command = line;
 	char help = 0;
 	for (int i = 0; i < CMD_NUM; i++) {
 		if (! strcmp(execmd, cmds[i].name)) {
@@ -343,7 +383,7 @@ int execute(char * line, char addhistory)
 }
 
 
-/* aptsh */
+/* Aptsh's specific commands. */
 
 int apt_dump_cfg()
 {
@@ -362,10 +402,10 @@ int apt_help()
 
 int apt_help_howto()
 {
-	char * cmdtmp = aptcmd;
-	aptcmd = trimleft(aptcmd)+strlen("help-howto");
+	char * cmdtmp = actual_command;
+	actual_command = trimleft(actual_command)+strlen("help-howto");
 	newcmd("zcat /usr/share/doc/aptsh/HOWTO.gz");
-	aptcmd = cmdtmp;
+	actual_command = cmdtmp;
 
 //	system("zcat /usr/share/doc/aptsh/howto.gz");
 //	realizecmd("man aptsh");
@@ -373,27 +413,6 @@ int apt_help_howto()
 	return 0;
 }
 
-/* We should ignore SIGPIPE.
- * If we don't, aptsh is going to terminate when user creates a bad pipe. 
- * Example:
- * rls | less
- * and user escapes from less before it reaches end of list.
- */
-void i_setsig()
-{
-	static char set = 0;
-	struct sigaction act;
-	if (! set) {
-		act.sa_handler = SIG_IGN;
-		act.sa_flags = 0;
-		sigaction(SIGPIPE, &act, NULL);
-		set = 1;
-	}
-}
-
-// This is used by apt_regex() and apt_ls().
-#define LIBEXEC_PREFIX "aptsh_"
-
 int apt_regex()
 {
 	FILE * pipe;
@@ -404,10 +423,10 @@ int apt_regex()
 
 	i_setsig();
 
-	// prevents from situation when user prefixes cmd with whitespaces
-	char * aptcmd_t = trimleft(aptcmd);
-	tmp = (char*)malloc(strlen(aptcmd_t)+strlen(SHARED_DIR)+strlen(LIBEXEC_PREFIX)+5);
-	sprintf(tmp, "%s%s%s%c", SHARED_DIR, LIBEXEC_PREFIX, aptcmd_t, '\0');
+	/* Prevent from situation when user prefixes cmd with whitespaces. */
+	char * actual_command_t = trimleft(actual_command);
+	tmp = (char*)malloc(strlen(actual_command_t)+strlen(SHARED_DIR)+strlen(LIBEXEC_PREFIX)+5);
+	sprintf(tmp, "%s%s%s%c", SHARED_DIR, LIBEXEC_PREFIX, actual_command_t, '\0');
 	pipe = popen(tmp, "w");
 	
 	while (e.end() == false) {
@@ -431,10 +450,10 @@ int apt_ls()
 
 	i_setsig();
 	
-	// prevents from situation when user prefixes cmd with whitespaces
-	char * aptcmd_t = trimleft(aptcmd);
-	tmp = (char*)malloc(strlen(aptcmd_t)+strlen(SHARED_DIR)+strlen(LIBEXEC_PREFIX)+4);
-	sprintf(tmp, "\%s%s%s%c", SHARED_DIR, LIBEXEC_PREFIX, aptcmd_t, '\0');
+	/* Prevents from situation when user prefixes cmd with whitespaces. */
+	char * actual_command_t = trimleft(actual_command);
+	tmp = (char*)malloc(strlen(actual_command_t)+strlen(SHARED_DIR)+strlen(LIBEXEC_PREFIX)+4);
+	sprintf(tmp, "\%s%s%s%c", SHARED_DIR, LIBEXEC_PREFIX, actual_command_t, '\0');
 	pipe = popen(tmp, "w");
 
 	while (e.end() == false) {
@@ -462,25 +481,25 @@ int apt_dpkg_reconfigure()
 
 int apt_whichpkg()
 {
-	char * cmdtmp = aptcmd;
-	aptcmd = trimleft(aptcmd)+strlen("whichpkg");
+	char * cmdtmp = actual_command;
+	actual_command = trimleft(actual_command)+strlen("whichpkg");
 	newcmd("dpkg -S");
-	aptcmd = cmdtmp;
+	actual_command = cmdtmp;
 	
 	return 0;
 }
 
 int apt_listfiles()
 {
-	char * cmdtmp = aptcmd;
-	aptcmd = trimleft(aptcmd)+strlen("listfiles");
+	char * cmdtmp = actual_command;
+	actual_command = trimleft(actual_command)+strlen("listfiles");
 	newcmd("dpkg -L");
-	aptcmd = cmdtmp;
+	actual_command = cmdtmp;
 
 	return 0;
 }
 
-// Display orphaned libraries in the system
+/* Display orphaned libraries in the system. */
 int apt_orphans()
 {
 	static pkgCache * Cache;
@@ -491,10 +510,22 @@ int apt_orphans()
 	e = Cache->PkgBegin();
 	i = 0;
 
+	/* FIXME: This probably causues a memleak (we lose control
+	 * on whitespaces before actual_command_t).
+	 */
+	char *actual_command_t = trimleft(actual_command);
+	char *fw = first_word(actual_command_t);
+	actual_command_t += strlen(fw);
+	//actual_command_t = trimleft(actual_command_t);
+	char *tmp = (char*)malloc(strlen(actual_command_t)+strlen(SHARED_DIR)+strlen(LIBEXEC_PREFIX)+strlen("printer ")+4);
+	sprintf(tmp, "\%s%s%s%s%c", SHARED_DIR, LIBEXEC_PREFIX, "printer ", actual_command_t, '\0');
+	free(fw);
+	FILE *pipe = popen(tmp, "w");
+
 	while (e.end() == false) {
 		if (e->CurrentVer != 0) {
 			const char * section = e.Section();
-			if (section == NULL) // it can't be library, since it doesn't belong to any section. :)
+			if (section == NULL) /* It can't be library, since it doesn't belong to any section. */
 				continue;
 			if (strstr(section, "libs") || strstr(section, "libdevel")) {
 				//printf("%s\n", e.Name());
@@ -512,7 +543,7 @@ int apt_orphans()
 					}
 				}
 				if (! found)
-					printf("%s\n", e.Name());
+					fprintf(pipe, "%s\n", e.Name());
 			}
 		}
 		/*if (! strncmp(e.Name(), text, len)) {
@@ -524,10 +555,13 @@ int apt_orphans()
 		e++;
 	}
 
+	pclose(pipe);
+	free(tmp);
+	
 	return 0;
 }
 
-// Display all orphaned (without any reverse dependencies installed) packages in the system
+/* Display all orphaned (without any reverse dependencies installed) packages in the system. */
 int apt_orphans_all()
 {
 	static pkgCache * Cache;
@@ -538,7 +572,19 @@ int apt_orphans_all()
 	e = Cache->PkgBegin();
 	i = 0;
 
-	column_display * view = new column_display(2, ' ');
+	/* FIXME: This probably causues a memleak (we lose control
+	* on whitespaces before actual_command_t).
+	*/
+	char *actual_command_t = trimleft(actual_command);
+	char *fw = first_word(actual_command_t);
+	actual_command_t += strlen(fw);
+	//actual_command_t = trimleft(actual_command_t);
+	char *tmp = (char*)malloc(strlen(actual_command_t)+strlen(SHARED_DIR)+strlen(LIBEXEC_PREFIX)+strlen("printer ")+4);
+	sprintf(tmp, "\%s%s%s%s%c", SHARED_DIR, LIBEXEC_PREFIX, "printer ", actual_command_t, '\0');
+	free(fw);
+	FILE *pipe = popen(tmp, "w");
+
+	column_display * view = new column_display(2, ' ', pipe);
 
 	while (e.end() == false) {
 		if (e->CurrentVer != 0) {
@@ -560,6 +606,9 @@ int apt_orphans_all()
 
 	view->dump();
 
+	pclose(pipe);
+	free(tmp);
+	
 	delete view;
 
 	return 0;
@@ -569,9 +618,9 @@ int apt_queue_commit()
 {
 	use_realcmd = 0;
 
-	for (int i = 0; i < commit_count; i++) {
-		printf(" >>> Doing step %d of %d...\n", i+1, commit_count);
-		execute(commitz[i], 0);
+	for (int i = 0; i < command_queue_count; i++) {
+		printf(" >>> Doing step %d of %d...\n", i+1, command_queue_count);
+		execute(command_queue_items[i], 0);
 	}
 
 	return 0;
@@ -580,15 +629,15 @@ int apt_queue_commit()
 int apt_queue_commit_say()
 {
 	
-	yes = trimleft(trimleft(aptcmd)+strlen("queue-commit-say"));
+	yes = trimleft(trimleft(actual_command)+strlen("queue-commit-say"));
 	
 	use_realcmd = 1;
 
 	i_setsig();
 
-	for (int i = 0; i < commit_count; i++) {
-		printf(" >>> Doing step %d of %d...\n", i+1, commit_count);
-		execute(commitz[i], 0);
+	for (int i = 0; i < command_queue_count; i++) {
+		printf(" >>> Doing step %d of %d...\n", i+1, command_queue_count);
+		execute(command_queue_items[i], 0);
 	}
 
 	use_realcmd = 0;
@@ -598,26 +647,26 @@ int apt_queue_commit_say()
 
 int apt_queue_clear()
 {
-	for (int i = 0; i < commit_count; i++) {
-		free(commitz[i]);
+	for (int i = 0; i < command_queue_count; i++) {
+		free(command_queue_items[i]);
 	}
-	free(commitz);
-	commitz = NULL;
-	commit_count = 0;
+	free(command_queue_items);
+	command_queue_items = NULL;
+	command_queue_count = 0;
 	return 0;
 }
 
-// removes item from commitz
+/* Removes item from 'command_queue_items'. */
 void real_remove(int num)
 {
 	int hm = num;
-	free(commitz[hm]);
-	while (hm < (commit_count-1)) {
-		commitz[hm] = commitz[hm+1];
+	free(command_queue_items[hm]);
+	while (hm < (command_queue_count-1)) {
+		command_queue_items[hm] = command_queue_items[hm+1];
 		hm++;
 	}
- 	commit_count--;
-	commitz = (char**)realloc(commitz, commit_count*sizeof(char*));
+ 	command_queue_count--;
+	command_queue_items = (char**)realloc(command_queue_items, command_queue_count*sizeof(char*));
 }
 
 int compare_ints(const void *a, const void *b)
@@ -630,20 +679,19 @@ int compare_ints(const void *a, const void *b)
 
 
 
-// Function below is used internally by apt_commit_remove() to add items to 
-// internal array of items to remove from command-queue
-#if __GNUC__ >= 3
-inline
-#endif
-void add_item(const int value, int * (&arr), int & arrlen)
+/* Function below is used internally by apt_commit_remove() to add items to 
+ * internal array of items to remove from command-queue.
+ */
+inline void add_item(const int value, int * (&arr), int & arrlen)
 {
-	if ((value > 0) && (value <= commit_count)) {
+	if ((value > 0) && (value <= command_queue_count)) {
 		/* Check whether item already exist */
 		bool exists = false;
 		for (int j = 0; j < arrlen; j++) {
 			if (arr[j] == value) {
-				/*It produces too much unnecessary mess, thus it's commented\
-				  fprintf(stderr, "Hey, you can't remove item %d twice!\n", value); */
+				/* It produces too much unnecessary mess, thus it's commented
+				 * fprintf(stderr, "Hey, you can't remove item %d twice!\n", value);
+				 */
 				exists = true;
 			}
 		}
@@ -658,7 +706,7 @@ void add_item(const int value, int * (&arr), int & arrlen)
 
 int apt_queue_remove()
 {
-	char * cmd = trimleft(aptcmd);
+	char * cmd = trimleft(actual_command);
 	char * tmp = first_word(cmd);
 	cmd = cmd+strlen(tmp);
 	int arrlen = 0;
@@ -676,17 +724,17 @@ int apt_queue_remove()
 			char * begin = (char*)malloc(len+1);
 			strncpy(begin, tmp, len);
 			begin[len] = '\0';
-			end++; // It contants '-', so we omit it...
+			end++; /* It contants '-', so we omit it. */
 			int _begin;
 			if (! strcmp(begin, "last")) {
-				_begin = commit_count;
+				_begin = command_queue_count;
 			} else {
 				_begin = atoi(begin);
 			}
 			
 			int _end;
 			if (! strcmp(end, "last")) {
-				_end = commit_count;
+				_end = command_queue_count;
 			} else {
 				_end = atoi(end);
 			}
@@ -701,10 +749,10 @@ int apt_queue_remove()
 				}
 			free(begin);
 		} else {
-			// add_item is a macro, so it would be converted MANY times without this
+			/* add_item is a macro, so it would be converted MANY times without this. */
 			int _tmp;
 			if (! strcmp(tmp, "last")) {
-				_tmp = commit_count;
+				_tmp = command_queue_count;
 			} else {
 				_tmp = atoi(tmp);
 			}
@@ -713,6 +761,9 @@ int apt_queue_remove()
 	}
 	free(tmp);
 	qsort(arr, arrlen, sizeof(int), compare_ints);
+	/* This is not too efficient - complexity of removing is about O(n^2).
+	 * It can be done with O(n) complexity, but O(n^2) should be enough.
+	 */
 	for (int i = arrlen-1; i >= 0; i--) {
 		real_remove(arr[i]-1);
 	}
@@ -722,13 +773,13 @@ int apt_queue_remove()
 
 int apt_queue()
 {
-	for (int i = 0; i < commit_count; i++) {
-		printf("%d: %s\n", i+1, commitz[i]);
+	for (int i = 0; i < command_queue_count; i++) {
+		printf("%d: %s\n", i+1, command_queue_items[i]);
 	}
 	return 0;
 }
 
-/* apt-get */
+/* Apt-get specific commands. */
 
 int apt_install()
 {
@@ -825,7 +876,7 @@ int apt_autoclean()
 	return 0;
 }
 
-/* apt-cache */
+/* Apt-cache specific commands. */
 
 int apt_show()
 {
@@ -919,9 +970,9 @@ int apt_madison()
 
 int apt_whatis()
 {
-#define WHATIS_CMD "apt-cache show %s | grep ^Description | head -n 1 | sed 's/Description://'"
-	char * tmp = (char*)malloc(strlen(aptcmd)+strlen(WHATIS_CMD));
-	sprintf(tmp, WHATIS_CMD, (char*)(trimleft(aptcmd)+strlen("whatis ")));
+	static char *whatis_shell_command = "apt-cache show %s | grep ^Description | head -n 1 | sed 's/Description://'";
+	char * tmp = (char*)malloc(strlen(actual_command)+strlen(whatis_shell_command));
+	sprintf(tmp, whatis_shell_command, (char*)(trimleft(actual_command)+strlen("whatis ")));
 	//system(tmp);
 	realizecmd(tmp);
 	free(tmp);
