@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005 Marcin R. Wrochniak <wrochniak@gmail.com>
+  Copyright (C) 2005-2006 Marcin R. Wrochniak <wrochniak@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2 as
@@ -41,8 +41,8 @@ void pkg_next(struct package * pkg);
 extern int hm;
 extern int hm_i;
 
-extern char ** pkgs; /* all packages */
-extern char ** pkgs_i; /* installed packages */
+extern char ** pkgs; /* All packages. */
+extern char ** pkgs_i; /* Only installed packages. */
 
 #ifdef __cplusplus
 }
